/*
  hardware.h - Hardware Library

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>

#ifdef ARDUINO
#include <Arduino.h>
#else
#define INPUT 0
#define OUTPUT 1
#define INPUT_PULLUP 2
#define HIGH 1
#define LOW  0
#endif

#define DEFAULT_STACK_SIZE 128

class Hardware
{
  public:

#ifdef ARDUINO
    static uint32_t micros(void) { return ::micros(); }
    static void pinMode(uint8_t pin, uint8_t mode) { ::pinMode(pin, mode); }
    static void digitalWrite(uint8_t pin, uint8_t val) { ::digitalWrite(pin, val); }
    static uint8_t digitalRead(uint8_t pin) { return ::digitalRead(pin); }
#else
    static uint32_t micros(void);
    static void pinMode(uint8_t pin, uint8_t mode);
    static void digitalWrite(uint8_t pin, uint8_t val);
    static uint8_t digitalRead(uint8_t pin);
#endif
    static size_t disableAndSaveInterrupts(void);
    static void restoreInterrupts(size_t save);
    static void idle(void);

  protected:

    /* For Alarm Class */
    static void setupAlarmTimer(void);
    static void startAlarmTimer(int32_t interval);
};
